/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.io.File;
import java.io.FileInputStream;
import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.PortletDiskFileUpload;
import org.jboss.logging.Logger;
import org.jboss.wiki.exceptions.EditSessionExpired;
import org.jboss.wiki.exceptions.EditingNotAllowedException;
import org.jboss.wiki.exceptions.WikiManagementNotFoundException;
import org.jboss.wiki.exceptions.WikiSaveException;
import org.jboss.wiki.management.WikiServiceManagement;

/**
 * The WikiPortlet.
 * <p>
 * 
 * @author <a href="mailto:dsicore@jboss.org">Damon Sicore</a>
 * @author <a href="mailto:tomasz.szymanski@jboss.com">Tomasz Szymanski</a>
 * @version $Id$
 */

public class WikiPortlet extends GenericPortlet {

	final static String jspPath = "/WEB-INF/jsp/";

	private static final String WINDOWS_FILE_NAME = "\\w:\\\\.+";

	private String defaultPage = "Main";

	private WikiEngine wikiEngine;

	private Logger log;

	public void init() {
		log = Logger.getLogger(WikiPortlet.class);

		WikiCommon wikiCommon = new WikiCommon();

		try {
			WikiServiceManagement wikiServiceMenagement = wikiCommon
					.getWikiManagement();
			wikiEngine = wikiServiceMenagement.getWikiEngine();
		} catch (WikiManagementNotFoundException e1) {
			log.error("Couldn't find WikiManagement", e1);
			wikiEngine = new WikiEngine();
		}
	}

	private Map<String, String> convertParameters(ActionRequest rReq) {
		Map<String, String> convertedParameters = new HashMap<String, String>();

		Map paramMap = rReq.getParameterMap();

		for (Object key : paramMap.keySet()) {
			try {
				if (rReq.getParameter("editAction") != null) {
					convertedParameters.put((String) key, new String(rReq
							.getParameter((String) key).getBytes("ISO-8859-1"),
							"UTF-8"));
				} else {
					convertedParameters.put((String) key, rReq
							.getParameter((String) key));
				}
			} catch (UnsupportedEncodingException e) {
				log.error(e);
				convertedParameters.put((String) key, (String) paramMap
						.get(key));
			}
		}

		return convertedParameters;
	}

	public void processAction(ActionRequest rReq, ActionResponse rResp) {

		Map<String, String> convertedParameters = convertParameters(rReq);

		Credentials credentials = wikiEngine.getCredentialsInstance(rReq);

		// wikiContext is null - to get refernece to actual page

		WikiContext wikiContext = null;

		if (PortletDiskFileUpload.isMultipartContent(rReq)) {
			PortletDiskFileUpload dfu = new PortletDiskFileUpload();

			// maximum allowed file upload size (10 MB)
			dfu.setSizeMax(10 * 1000 * 1000);

			// maximum size in memory (vs disk) (100 KB)
			dfu.setSizeThreshold(100 * 1000);

			try {
				// get the FileItems
				List fileItems = dfu.parseRequest(rReq);
				Iterator iter = fileItems.iterator();
				while (iter.hasNext()) {
					FileItem item = (FileItem) iter.next();
					if (item.isFormField()) {
						// pass along to render request
						String fieldName = item.getFieldName();
						String value = item.getString();
						rResp.setRenderParameter(fieldName, value);
					} else {
						// write the uploaded file to a new location
						String fieldName = item.getFieldName();
						String fileName = item.getName();

						// change windows names (remove c:\bull\shit\)
						if (fileName.matches(WINDOWS_FILE_NAME)) {
							fileName = fileName.substring(fileName
									.lastIndexOf('\\') + 1, fileName.length());
						}

						if (fileName.length() == 0) {
							throw new FileUploadException("Wrong file name: "
									+ item.getName());
						}

						// delete spaces

						while (fileName.indexOf(' ') != -1) {
							Character chr = fileName.charAt(fileName
									.indexOf(' ') + 1);

							fileName = fileName.substring(0, fileName
									.indexOf(' '))
									+ (char) Character.toUpperCase(chr)
									+ fileName.substring(
											fileName.indexOf(' ') + 2, fileName
													.length());
						}

						String tempDir = System.getProperty("java.io.tmpdir");
						String serverFileName = fieldName + fileName
								+ "-portlet.tmp";

						File serverFile = new File(tempDir, serverFileName);

						item.write(serverFile);

						log.info("serverFileName : " + tempDir + "/"
								+ serverFileName + " filename: " + fileName);

						WikiPage pageWithAtt = wikiEngine.getByName(
								convertedParameters.get("page"), null,
								(String) convertedParameters.get("language"));

						FileInputStream fis = new FileInputStream(serverFile);

						pageWithAtt.addAttachement(fis, fileName, credentials
								.getName());

						// at the end delete the temp file
						serverFile.delete();
					}
				}
			} catch (FileUploadException fue) {
				String msg = "File Upload Exception: " + fue.getMessage();
				rResp.setRenderParameter("error", msg);
				getPortletContext().log(msg, fue);
			} catch (Exception e) {
				String msg = "Exception: " + e.getMessage();
				rResp.setRenderParameter("error", msg);
				getPortletContext().log(msg, e);
			}
		}

		if (convertedParameters.get("page") != null
				&& (convertedParameters.get("page").length() > 0)) {
			StringBuffer sBuf = new StringBuffer(convertedParameters
					.get("page"));

			if (Character.isLowerCase(sBuf.charAt(0))) {
				// change first letter to upper if it isn't
				sBuf.setCharAt(0, Character.toUpperCase(sBuf.charAt(0)));
			}

			//
			// Capitalise words in subpages
			//

			for (int i = 0; i < sBuf.length() - 1; i++) {
				if ((sBuf.charAt(i) == '/')
						&& (Character.isLowerCase(sBuf.charAt(i + 1)))) {
					sBuf.setCharAt(i + 1, Character.toUpperCase(sBuf
							.charAt(i + 1)));
				}
			}

			rResp.setRenderParameter("page", sBuf.toString());

		}

		if (convertedParameters.get("searchQuery") != null) {
			Map<String, Integer> pages = wikiEngine.findPages(
					convertedParameters.get("searchQuery"),
					(String) convertedParameters.get("language"));

			rReq.getPortletSession().setAttribute("findResult", pages);

			// show the results
			rResp.setRenderParameter("page", "FindResult");
		}

		int i = 1;

		while (convertedParameters.get("var" + i) != null) {
			rReq.getPortletSession().setAttribute("var" + i,
					convertedParameters.get("var" + i));
			i++;
		}

		if (convertedParameters.get("action") != null) {
			rResp.setRenderParameter("action", convertedParameters
					.get("action"));
		}

		if (convertedParameters.get("type") != null) {
			rResp.setRenderParameter("type", convertedParameters.get("type"));
		}

		if (convertedParameters.get("version") != null) {
			log.info("[process action]Getting page "
					+ convertedParameters.get("page") + " at version "
					+ convertedParameters.get("version"));
			rResp.setRenderParameter("version", convertedParameters
					.get("version"));
		}

		if (convertedParameters.get("language") != null) {
			rResp.setRenderParameter("language", (String) convertedParameters
					.get("language"));
		} else {
			rResp.setRenderParameter("language", wikiEngine.defaultLangugeCode);
		}

		if (convertedParameters.get("editAction") != null) {
			log.debug("EDITING PAGE");

			String editedPageName = convertedParameters.get("editedPage");

			// unlock page
			WikiPage edPage = wikiEngine.getByName(editedPageName, wikiContext,
					(String) convertedParameters.get("language"));

			if (edPage != null) { // it may be null if the page isn't saved
				// yet
				edPage.unlock(credentials, Integer
						.valueOf((String) convertedParameters
								.get("editSessionId")));
			}

			// saving, previewing or canceling

			if (convertedParameters.get("editAction").equals("Save")) {

				// save the page

				if (edPage == null) {
					edPage = new WikiPage(editedPageName, credentials, "", 0,
							0, new Date(), wikiEngine, true, true, null, null,
							0, (String) convertedParameters.get("language"));
				}

				// set new content
				edPage.setContent(convertedParameters.get("wikiContent"));

				// set author name for new version
				edPage.setLastAuthor(credentials);

				// increase page version
				edPage.setLastVersion(edPage.getLastVersion() + 1);

				// set new data
				edPage.setEditDate(new Date());

				// save the page
				try {
					edPage.save(credentials, Integer
							.valueOf((String) convertedParameters
									.get("editSessionId")),
							(String) convertedParameters.get("language"),
							new PortletWikiSession(rReq.getPortletSession()));
				} catch (NumberFormatException e) {
					log.error(e);
				} catch (EditSessionExpired e) {
					rReq.setAttribute("errorMsg", e.getMessage());
					log.error(e);
				} catch (WikiSaveException e) {
					rReq.setAttribute("errorMsg", e.getMessage());
					log.error(e);
				} catch (EditingNotAllowedException e) {
					rReq.setAttribute("errorMsg", e.getMessage());
					log.error(e);
				}

				// and finally remove old page object from memory to force
				// reloading it
				wikiEngine.refreshPage(edPage.getName(), edPage.getLangCode());
			}
		}

	}

	public void doView(RenderRequest rReq, RenderResponse rRes)
			throws PortletException, java.io.IOException {
		try {
			WikiSession wikiSession = new PortletWikiSession(rReq
					.getPortletSession());

			Credentials credentials = wikiEngine.getCredentialsInstance(rReq);

			String langCode = rReq.getParameter("language");
			
			if (langCode == null) {
				langCode = wikiEngine.defaultLangugeCode;
			}

			String wikiURL = rRes.createActionURL().toString() + "&language="
					+ langCode;

			rReq.setAttribute("actionURL", wikiURL);

			if (langCode == null) {
				langCode = wikiEngine.defaultLangugeCode;
			}

			boolean showPage = true;

			// Create server host and put it into session for plugins
			String hostURL = ((rReq.isSecure()) ? "https" : "http")
					+ "://"
					+ rReq.getServerName()
					+ ((rReq.getServerPort() == 80) ? "" : ":"
							+ rReq.getServerPort());

			prepareSession(rReq.getPortletSession(), credentials, hostURL, wikiURL, rReq.isSecure());

			/*
			 * WikiPage noSuchPage = new WikiPage("", credentials, "There is no
			 * such page. Click on EDIT to start it.", 0, 0, null,
			 * wikiEngine.getMediaDataSource(), wikiEngine
			 * .getAttachementDataSource(), true, true);
			 */

			String page = "Wiki.jsp";

			String wikiPage = defaultPage;

			WikiPage pageToShow = null;

			WikiContext wikiContext = null;

			if (rReq.getParameter("page") != null) {
				wikiPage = rReq.getParameter("page");
			}

			if ((rReq.getParameter("action") != null)
					&& (rReq.getParameter("action").equals("Attach"))) {
				rReq.setAttribute("fileAttach", "true");
			}

			if ((rReq.getParameter("action") != null)
					&& (rReq.getParameter("action").equals("Edit"))
					&& (credentials.isLoggedIn())) {
				page = "Edit.jsp";
				showPage = false;

				WikiContext editWikiContext = new WikiContext(credentials,
						wikiEngine.getWikiType("edit"), wikiSession);

				pageToShow = wikiEngine.getByName(wikiPage, editWikiContext,
						langCode);

				if ((pageToShow != null)
						&& (!pageToShow.isEditable(credentials))) {
					if (!credentials.isAdmin()) {
						showPage = true;
					}
				} else {
					if (rReq.getParameter("version") != null) {
						// show previous version of page

						log.info("Getting page " + wikiPage + " at version "
								+ rReq.getParameter("version"));

						if ((pageToShow.getLastVersion() > Integer.valueOf(rReq
								.getParameter("version")))) {
							pageToShow = wikiEngine
									.getByName(wikiPage, editWikiContext,
											Integer.valueOf(rReq
													.getParameter("version")),
											langCode);
						}

						if (pageToShow.getLastVersion() != pageToShow
								.getVersion()) {
							rReq.setAttribute("showedVersion", Integer
									.valueOf(rReq.getParameter("version")));
						}
					} else {
						pageToShow = wikiEngine.getByName(wikiPage,
								editWikiContext, langCode);
					}

					if (pageToShow == null) {
						// start editing new page
						// wikiPage = defaultPage;

						pageToShow = new WikiPage(wikiPage, credentials, "", 0,
								0, new Date(), wikiEngine, true, true, null,
								null, 0, langCode);
						// wikiEngine.getByName(wikiPage, wikiContext);
					}

					int editSessionId = wikiEngine.getEditSessionId();

					Date dateLock = wikiEngine.getByName(wikiPage, null,
							langCode).lock(credentials, editSessionId);

					rReq.setAttribute("editSessionId", String
							.valueOf(editSessionId));

					if (dateLock != null) {
						rReq
								.setAttribute(
										"lockMsg",
										"Page is locked by "
												+ pageToShow.getLockUser()
												+ " from "
												+ dateLock
												+ ".<br> I won't stop you from saving, but you or he can erease each other changes. Lock will expire: ");
					}
				}
			} else if ((rReq.getParameter("action") != null)
					&& (rReq.getParameter("action").equals("Info"))) {
				page = "PageInfo.jsp";
				showPage = false;

				pageToShow = wikiEngine.getByName(wikiPage, null, langCode);

				/*
				 * if (pageToShow == null) { // if somebody entered a link to
				 * info // non-existing page page = "Wiki.jsp"; try { pageToShow =
				 * (WikiPage) noSuchPage.clone(); } catch
				 * (CloneNotSupportedException cnse) { log.info(cnse); }
				 * 
				 * pageToShow.setName(wikiPage); } else {
				 */
				rReq.setAttribute("pageAtTop", pageToShow);

			}

			if (showPage) {
				page = "Wiki.jsp";

				String type = "html";

				if (rReq.getParameter("type") != null) {
					type = rReq.getParameter("type");
					// log.info("Getting type: " + type);
				}

				wikiContext = new WikiContext(credentials, wikiEngine
						.getWikiType(type), new PortletWikiSession(rReq
						.getPortletSession()));

				if (rReq.getParameter("version") != null) {
					// show previous version of page
					log.info("Getting page " + wikiPage + " at version "
							+ rReq.getParameter("version"));
					pageToShow = wikiEngine.getByName(wikiPage, wikiContext,
							Integer.valueOf(rReq.getParameter("version")),
							langCode);

					if (pageToShow.getLastVersion() != pageToShow.getVersion()) {
						rReq.setAttribute("showedVersion", Integer.valueOf(rReq
								.getParameter("version")));
					}
				} else {
					pageToShow = wikiEngine.getByName(wikiPage, wikiContext,
							langCode);
				}

				/*
				 * if (pageToShow == null) {
				 * 
				 * try { pageToShow = (WikiPage) noSuchPage.clone(); } catch
				 * (CloneNotSupportedException cnse) { log.info(cnse); }
				 * 
				 * pageToShow.setName(wikiPage); }
				 */

			}

			rReq
					.setAttribute(
							"wikiName",
							"<img src=\""
									+ hostURL
									+ "/wiki/"
									+ WikiCommon.wikiButtons
									+ "/JBossWikiLogoMedium.png\" width=\"100\"  border=\"0\" style=\"vertical-align: middle\"/>");

			rReq.setAttribute("wikiContent", pageToShow.getContent());

			rReq.setAttribute("wikiImagesUrl", hostURL + "/wiki/"
					+ WikiCommon.wikiButtons + "/");

			rReq.setAttribute("wikiPage", pageToShow.getName());

			rReq.setAttribute("editDate", pageToShow.getEditDate());

			rReq.setAttribute("author", pageToShow.getLastAuthor());

			rReq.setAttribute("logedIn", credentials.isLoggedIn());

			rReq.setAttribute("defaultPage", defaultPage);

			rReq.setAttribute("isEditable", pageToShow.isEditable(credentials));

			rReq.setAttribute("isViewable", pageToShow.isViewable(credentials));

			rReq.setAttribute("isAdmin", credentials.isAdmin());

			rReq.setAttribute("usedLangs", wikiEngine.getUsedLanguageCodes());

			rReq.setAttribute("currentLanguage", langCode);

			rReq
					.setAttribute(
							"friendlyLink",
							hostURL
									+ "/wiki/"
									+ ((!langCode
											.equals(wikiEngine.defaultLangugeCode)) ? langCode
											.toLowerCase()
											+ "/"
											: "") + pageToShow.getName());
			rReq
					.setAttribute(
							"rssLink",
							hostURL
									+ "/wiki/"
									+ ((!langCode
											.equals(wikiEngine.defaultLangugeCode)) ? langCode
											.toLowerCase()
											+ "/"
											: "") + "rss/"
									+ pageToShow.getName());
			rReq
					.setAttribute(
							"rdfLink",
							hostURL
									+ "/wiki/"
									+ ((!langCode
											.equals(wikiEngine.defaultLangugeCode)) ? langCode
											.toLowerCase()
											+ "/"
											: "") + "rdf/"
									+ pageToShow.getName());
			rReq
					.setAttribute(
							"atomLink",
							hostURL
									+ "/wiki/"
									+ ((!langCode
											.equals(wikiEngine.defaultLangugeCode)) ? langCode
											.toLowerCase()
											+ "/"
											: "") + "atom/"
									+ pageToShow.getName());

			rRes.setTitle("Wiki");
			if (javax.portlet.WindowState.NORMAL.equals(rReq.getWindowState())
					|| javax.portlet.WindowState.MAXIMIZED.equals(rReq
							.getWindowState())) {
				rRes.setContentType("text/html; charset=UTF-8");
				javax.portlet.PortletRequestDispatcher pRD = this
						.getPortletContext().getRequestDispatcher(
								jspPath + page);
				pRD.include(rReq, rRes);
				return;
			}
			doDispatch(rReq, rRes);
		} catch (RuntimeException e) {
			e.printStackTrace();
			throw (e);
		}
	}

	private void prepareSession(PortletSession portletSession,
			Credentials credentials, String hostURL, String wikiURL, Boolean isSecure) {
		// prepare some data for plugins in session
		portletSession.setAttribute(WikiSession.CREDENTIALS, credentials);
		portletSession.setAttribute(WikiSession.HOST_URL, hostURL);
		portletSession.setAttribute(WikiSession.ACTION_URL, wikiURL);
		portletSession.setAttribute(WikiSession.IS_SECURE, isSecure);
	}
}