/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.portlet.ActionRequest;
import javax.portlet.RenderRequest;
import javax.transaction.TransactionManager;

import org.jboss.logging.Logger;
import org.jboss.portal.common.transaction.TransactionException;
import org.jboss.portal.common.transaction.TransactionManagerProvider;
import org.jboss.portal.common.transaction.Transactions;
import org.jboss.portal.core.modules.ModuleException;
import org.jboss.portal.identity.IdentityException;
import org.jboss.portal.identity.Role;
import org.jboss.portal.identity.RoleModule;
import org.jboss.portal.identity.MembershipModule;
import org.jboss.portal.identity.User;
import org.jboss.portal.identity.UserProfileModule;
import org.jboss.portal.identity.UserModule;
import org.jboss.portlet.JBossActionRequest;
import org.jboss.portlet.JBossRenderRequest;

public class JBossPortalCredentials extends Credentials {

  private final String adminRoleName = "Admin";

  private String name = null;

  private boolean logedIn = false;

  private boolean isAdmin = false;

  private Set<String> roles;

  private Set<Role> allRoles;

  private TransactionManager tm;

  private String email;

  private static final String userModuleCtx = "java:portal/UserModule";
  private static final String userProfileModuleCtx = "java:/portal/UserProfileModule";

  private static final String roleModulesCtx = "java:portal/RoleModule";

  private static final String membershipModulesCtx = "java:portal/MembershipModule";

  private Logger log = Logger.getLogger(this.getClass());

  public JBossPortalCredentials() {

  }

  public void setRenderRequest(RenderRequest rReq) {

  }

  public static Set<String> getRoleNames(User user, MembershipModule membershipModule) {

    Set<String> ret = new HashSet<String>();
    try {
      for (Object roleObj : membershipModule.getRoles(user)) {
        ret.add(((Role) roleObj).getName());
      }
    } catch (IdentityException e) {
      return ret;
    }

    return ret;
  }

  public void setParam(Object param) {

    InitialContext initialContext;
    RoleModule roleModule = null;
    UserModule userModule = null;
    MembershipModule membershipModule = null;
    UserProfileModule userProfileModule = null;

    try {
      initialContext = new InitialContext();
      roleModule = (RoleModule) initialContext
          .lookup(roleModulesCtx);
      membershipModule = (MembershipModule) initialContext
          .lookup(membershipModulesCtx);
      userModule = (UserModule) initialContext
          .lookup(userModuleCtx);
      userProfileModule = (UserProfileModule) initialContext
          .lookup(userProfileModuleCtx);
    } catch (NamingException e) {
      log.error(e);

      // should throw an exception here
      return;
    }


    if (param instanceof JBossActionRequest) {
      JBossActionRequest jrReq = (JBossActionRequest) param;

      if (jrReq.getUser() != null) {
        extractCredentialsFromUser(jrReq.getUser(),
                                   userModule,
                                   membershipModule,
                                   userProfileModule);
      }
    } else if (param instanceof JBossRenderRequest) {
      JBossRenderRequest jrReq = (JBossRenderRequest) param;

      if (jrReq.getUser() != null) {
        extractCredentialsFromUser(jrReq.getUser(),
                                   userModule,
                                   membershipModule,
                                   userProfileModule);
      } else {
        name = "Unknown";
      }
    } else {
      log.error("You are not running JBoss Portal");
    }

    try {
      allRoles = roleModule.findRoles();
    } catch (IdentityException e) {
      log.error("Couldn't get roles from portal", e);
    }
  }

  public String getName() {
    return name;
  }

  public boolean isAdmin() {
    return isAdmin;
  }

  public boolean isLoggedIn() {
    return logedIn;
  }

  @Override
  public boolean isUserInRole(String role) {
    if (roles != null)
      return roles.contains(role);
    else
      return false;
  }

  @Override
  public Set<String> getAllRoles() {
    Set<String> roleNames = new TreeSet<String>();

    for (Role role : allRoles) {
      roleNames.add(role.getName());
    }

    return roleNames;
  }

  @Override
  public String getEmail() {
    return email;
  }

  /**
   * Extract attributes from the current user and set the data as instance variables.
   * If the user passed to this method is null, nothing is extracted and the
   * methods just returns.
   * @param user the current user from which data is extracted
   * @param userModule the user module
   * @param membershipModule the membership module
   * @param userProfileModule the user profile module
   */
  private void extractCredentialsFromUser(User user,
                                          UserModule userModule,
                                          MembershipModule membershipModule,
                                          UserProfileModule userProfileModule) {
    if (user != null) {

      // Workaround: The membership module of the JBoss Portal 2.6 does not accept
      // the User instance from the JBossActionRequest or JBossRenderRequest.
      // Therefore convert it to an instance the membership module will accept.
      // See http://jira.jboss.com/jira/browse/JBPORTAL-1570
      User hibernateUser = null;
      try {
        hibernateUser = userModule.findUserById(user.getId());
      } catch (IdentityException e) {
        log.error("Couldn't convert cached user to Hibernate user.", e);
        return;
      }

      try {
        email = (String) userProfileModule.getProperty(user, User.INFO_USER_EMAIL_REAL);
      } catch (IdentityException e) {
        log.error("Couldn't get email from user profile module for user with ID: " + hibernateUser.getId());
        return;
      }

      name = user.getUserName();
      logedIn = true;
      isAdmin = getRoleNames(hibernateUser, membershipModule).contains(adminRoleName);
      roles = getRoleNames(hibernateUser, membershipModule);
    }
  }

}
