/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.wiki.plugins;

import java.util.Map;
import java.util.Set;

import org.jboss.wiki.Credentials;
import org.jboss.wiki.WikiPage;
import org.jboss.wiki.WikiPlugin;
import org.jboss.wiki.WikiSession;

public class LinkPagePlugin extends WikiPlugin {

	private final static String LINK_CAPTION = "Link";

	@Override
	public WikiPage process(WikiPage wikiPage, WikiSession wikiSession) {

		// you can link a page if it is editable for you
		if (wikiPage.isEditable((Credentials) wikiSession
				.getAttribute(WikiSession.CREDENTIALS))) {

			StringBuilder content = new StringBuilder(
					HTMLTranslatorParts.pagePartRegexStart);

			// process any link requests
			if (wikiSession.getAttribute("var1") != null
					&& wikiSession.getAttribute("var1").equals(LINK_CAPTION)) {
				String newLang = (String) wikiSession.getAttribute("var2");
				String pageName = (String) wikiSession.getAttribute("var3");

				WikiPage linkTo = wikiEngine.getByName(pageName, null, newLang);

				if (linkTo.isEditable((Credentials) wikiSession
						.getAttribute(WikiSession.CREDENTIALS))) {

					// make link in page we link to
					synchronized (linkTo) {
						makePageLinks(linkTo, wikiPage);
					}

					// make link in current page
					synchronized (wikiPage) {
						makePageLinks(wikiPage, linkTo);
					}

					content.append("Link added\n");
				} else {
					content
							.append("Page you're trying to link to is not editable for you.\n");
				}
			}

			// show Link page options
			content.append("!!Link page: ").append(wikiPage.getName()).append(
					" (").append(wikiPage.getLangCode()).append(")\n\n");

			content.append(HTMLTranslatorParts.pagePartRegexEnd);

			Map<String, String> usedLangs = wikiEngine.getUsedLanguageCodes();

			// iterate thru all langs used in this wiki
			for (String lang : usedLangs.keySet()) {
				if (!lang.equals(wikiPage.getLangCode())) {
					// create forms
					content.append(lang).append(" (").append(
							usedLangs.get(lang)).append(")<br />\n");
					content
							.append("<form method=\"post\" action=\"")
							.append(
									wikiSession
											.getAttribute(WikiSession.ACTION_URL))
							.append("&page=")
							.append(wikiPage.getName())
							.append("&type=")
							.append(wikiType.getName())
							.append(
									"\"><input type=\"hidden\" name=\"var2\" value=\"")
							.append(lang).append("\" /><select name=\"var3\">");

					Set<String> allPageNames = wikiEngine.getAllPageNames(lang);

					// page you can link to
					for (String pageName : allPageNames) {
						content.append("<option>").append(pageName).append(
								"</option>\n");
					}

					content.append("</select>").append(
							"<input type=\"submit\" value=\"").append(
							LINK_CAPTION).append("\" name=\"var1\" />");

					content.append("</form><br />\n");
				}
			}
			// copy page, as this is directref wikitype
			try {
				wikiPage = (WikiPage) wikiPage.clone();
			} catch (CloneNotSupportedException e) {
				log.error(e);
			}

			wikiPage.setContent(content.toString());

		} else {
			wikiPage.setContent("This page is not editable for you");
		}

		return wikiPage;
	}

	@Override
	public void init() {
		// TODO Auto-generated method stub

	}

	private void makePageLinks(WikiPage inPage, WikiPage toPage) {
		String links[] = { null };

		if (inPage.getMetaDataProperty(WikiPage.OTHER_LANGS) != null
				&& inPage.getMetaDataProperty(WikiPage.OTHER_LANGS).length() > 0) {

			links = inPage.getMetaDataProperty(WikiPage.OTHER_LANGS).split(
					"[,]");
		}

		boolean added = false;

		// look if there already is link to this lang
		for (int i = 0; links[0] != null && !added && i < links.length; i += 2) {
			if (links[i].equals(toPage.getLangCode())) {
				// remove link from previous page
				WikiPage oldLinkPage = wikiEngine.getByName(wikiEngine
						.getRealName(links[i + 1], links[i]), null, links[i]);

				removeOldLink(oldLinkPage, inPage);

				links[i + 1] = wikiEngine.getUid(toPage.getName(), toPage
						.getLangCode());
				added = true;
			}
		}

		// there were no link in this lang yet
		if (!added) {
			String langsStr = inPage.getMetaDataProperty(WikiPage.OTHER_LANGS);

			if (langsStr == null) {
				langsStr = "";
			} else {
				langsStr += ",";
			}

			inPage
					.setMetaDataProperty(WikiPage.OTHER_LANGS, langsStr
							+ toPage.getLangCode()
							+ ","
							+ wikiEngine.getUid(toPage.getName(), toPage
									.getLangCode()));
		} else {
			StringBuilder linksBuilder = new StringBuilder(links[0]);

			for (int i = 1; i < links.length; i++) {
				linksBuilder.append(",").append(links[i]);
			}

			inPage.setMetaDataProperty(WikiPage.OTHER_LANGS, linksBuilder
					.toString());
		}
	}

	private void removeOldLink(WikiPage oldLinkPage, WikiPage toPage) {
		if (oldLinkPage.getMetaDataProperty(WikiPage.OTHER_LANGS) != null) {
			synchronized (oldLinkPage) {
				String[] otherLangs = oldLinkPage.getMetaDataProperty(
						WikiPage.OTHER_LANGS).split("[,]");

				String newLangs = "";

				for (int i = 0; i < otherLangs.length; i += 2) {
					if (!otherLangs[i].equals(toPage.getLangCode())) {
						newLangs += "," + otherLangs[i] + ","
								+ otherLangs[i + 1];
					}
				}

				if (newLangs.length() > 1) {
					newLangs = newLangs.substring(1);
				}

				if (newLangs.length() > 0) {
					oldLinkPage.setMetaDataProperty(WikiPage.OTHER_LANGS,
							newLangs);
				} else {
					oldLinkPage.removeMetaDataProperty(WikiPage.OTHER_LANGS);
				}
			}
		}
	}
}
